"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/page_loader.py $
$Id: page_loader.py 27878 2006-01-26 20:09:07Z dbinger $
"""
from cStringIO import StringIO
from dulcinea.common import get_request, get_connection
from dulcinea.common import set_test_environment, get_publisher
from dulcinea.site_util import get_root_directory
from dulcinea.ui.crumbs import get_exports
from popen2 import popen2
from pprint import pformat
from sancho.utest import UTest
import sys


class PageLoader (object):

    def set_test_environment(self, **kwargs):
        return set_test_environment(**kwargs)

    def __init__(self):
        get_connection()
        self.set_test_environment()

        session_cookie_name = get_publisher().get_session_cookie_name()
        self.session_key = "test"
        self.cookies='%s="%s"' % (session_cookie_name, self.session_key)
        self.set_test_environment(HTTP_COOKIE=self.cookies)

        session = get_publisher().session_manager._create_session()

        self.user = self.get_admin_user()
        session.set_authenticated(self.user)
        session.id = self.session_key
        get_publisher().get_sessions()[self.session_key] = session
        self.session = session

    def process(self,
                PATH_INFO='/',
                input_string = '',
                SCRIPT_NAME='',
                SERVER_NAME='test@example.org',
                **kwargs):
        env = dict(PATH_INFO=PATH_INFO,
                   SCRIPT_NAME=SCRIPT_NAME,
                   SERVER_NAME=SERVER_NAME,
                   **kwargs)
        get_publisher().process(StringIO(input_string), env)
        request = get_request()
        return request

    def get_admin_user(self):
        for user in get_publisher().get_users().itervalues():
            if user.is_admin():
                return user
        return None

    def get_paths(self, user):
        def get_paths_inner(directory, base_path):
            get_request().environ['PATH_INFO'] = base_path + '/'
            exports = [export[0] for export in get_exports(directory)]
            exports.sort()
            for name in exports:
                objname = directory._q_translate(name)
                if objname is None:
                    obj = directory._q_lookup(name)
                else:
                    obj = getattr(directory, objname)
                if hasattr(obj, '_q_traverse'):
                    for path in get_paths_inner(obj, base_path + '/' + name):
                        yield path
                else:
                    yield base_path +'/' + name
        set_test_environment()
        self.session.set_authenticated(user)
        get_request().session = self.session
        return [path for path in get_paths_inner(get_root_directory(), '')]

    def get_processed_request(self, path, follow_redirects=(301,),
                               **kwargs):
        """
        Returns a request after processing, after one redirect if necessary.
        """
        if 'HTTP_COOKIE' not in kwargs:
            kwargs['HTTP_COOKIE'] = self.cookies
        self.session.set_authenticated(self.user)
        get_publisher().get_sessions()[self.session_key] = self.session
        request = self.process(path, **kwargs)
        if request.response.get_status_code() in follow_redirects:
            location = request.response.get_header('location')
            path = '/' + location.split('/', 3)[-1] # strip http://server
            self.session.set_authenticated(self.user)
            get_publisher().get_sessions()[self.session_key] = self.session
            request = self.process(path, **kwargs)
        return request

    def is_reportable(self, line):
        if 'Warning: ' in line:
            warning = line.split('Warning: ', 1)[1].strip()
            if warning in (
                '<table> lacks "summary" attribute',
                '<a> attribute "href" lacks value',
                '<form> attribute "action" lacks value',
                '<input> attribute "name" lacks value',
                'trimming empty <option>',
                'trimming empty <span>',
                '<textarea> proprietary attribute "wrap"',
                '<a> cannot copy name attribute to id', # starts with digit
                ):
                return False
            if warning.startswith("trimming empty <"):
                return False
            if (warning.endswith('" already defined') and
                warning.startswith('<a> anchor "')):
                return False
        return True

    def expect(self, path, status=200, **kwargs):
        get_publisher().logger.error_log = StringIO()
        request = self.get_processed_request(path, **kwargs)
        try:
            assert request.response.get_status_code() == status, (
                "%r (expected %r) %s" % (request.response.get_status_code(),
                                         status,
                                         path))
        except:
            print request.response.get_status_code(), path
            request.response.write(sys.stdout)
            if request.response.get_status_code() != status:
                print get_publisher().logger.error_log.getvalue()
            raise
        return request

    def validate_html(self, request, assert_valid=True):
        if request.response.get_content_type() == 'text/html':
            s = StringIO()
            for chunk in request.response.generate_body_chunks():
                s.write(chunk)
            ins, outs = popen2("/usr/bin/env tidy -q -e 2>&1")
            outs.write(s.getvalue())
            outs.close()
            body_lines = s.getvalue().splitlines()
            errors = []
            for line in ins.readlines():
                if self.is_reportable(line):
                    splitline = line.split()
                    lineno = int(splitline[1])
                    colno = int(splitline[3])
                    if body_lines:
                        bodyline = body_lines[lineno-1]
                    else:
                        bodyline = ''
                    errors.append(
                        '%s    ->%s: %r\n\n' % (
                        line,
                        lineno,
                        bodyline[max(colno-20, 0):colno+100]))
            if errors:
                print (
                    "\nLoading %r:\n%s\n\n" % (
                    request.get_path(), ''.join(errors)))
                if assert_valid:
                    assert not errors, 'html errors found'
        return request

    def crawl(self, user, omit_status=(200,)):
        """
        Collect information to use in a more general test.
        """
        expect = {}
        for path in self.get_paths(user):
            request = self.get_processed_request(path)
            if request.response.get_status_code() not in omit_status:
                expect[path] = request.response.get_status_code()
        assert 0, 'expect=\n%s' % pformat(expect)


class PageTest(UTest):

    def __init__(self):
        loader = PageLoader()
        self.__class__._expect = loader.expect
        self.__class__._get_admin_user = loader.get_admin_user
        self.__class__._get_paths = loader.get_paths
        UTest.__init__(self)



